# -*- coding: utf-8 -*-

__author__ = 'XQ'

import functools
import inspect
import re


def under_score_case(camel_case):
    if not camel_case:
        return ""
    return re.compile('([^A-Z])([A-Z])').sub(r"\1_\2", camel_case).lower()


class Decorator:
    def get_modules(self):
        if not self.__dict__.get("modules"):
            self.modules = {}
        return self.modules

    def module(self, module):
        modules = self.get_modules()
        stack = inspect.stack()
        cls = stack[1][3]
        is_class = cls != "<module>"
        _module = stack[1][0].f_locals.get("__module__" if is_class else "__name__")
        name = module

        def decorator(origin):
            metas = modules.get(_module)
            if metas:
                del modules[_module]
                modules[under_score_case(name)] = metas

            @functools.wraps(origin)
            def wrapper(*args, **kw):
                return origin(*args, **kw)

            return wrapper

        if isinstance(module, str):
            return decorator
        else:
            name = module.__name__
            return decorator(module)

    def action(self, action):
        modules = self.get_modules()
        stack = inspect.stack()
        cls = stack[1][3]
        is_class = cls != "<module>"
        _module = stack[1][0].f_locals.get("__module__" if is_class else "__name__")
        name = action

        def decorator(origin):
            @functools.wraps(origin)
            def wrapper(*args, **kw):
                return origin(*args, **kw)

            _meta = {
                "action": under_score_case(name),
                "module": _module,
                "method": origin.__name__,
            }
            if is_class:
                _meta["class"] = cls
            metas = modules.get(_module)
            if not metas:
                metas = []
                modules[_module] = metas
            params = list(origin.__code__.co_varnames)
            if is_class:
                _meta["class"] = cls
            if len(params) > 1 or not is_class and len(params) > 0:
                _meta["params"] = params
            metas.append(_meta)
            _doc = inspect.getdoc(origin)
            _description = _doc.split("\n")[0] if _doc else ""
            wrapper.__description__ = _description
            return wrapper

        if isinstance(action, str):
            return decorator
        name = action.__name__
        return decorator(action)



# -*- coding: utf-8 -*-

__author__ = 'XQ'

import os
import pkgutil
import importlib
import functools


def iter_package(package, filter=lambda _name: True):
    """
    迭代某package下所有子module
    :param package:包名或者包"core.crypt"
    :param filter:过滤字段,如handler,service,model等
    """
    if not package:
        return
    if type(package) is not str:
        package_name = package.__name__
    else:
        package_name = package
    package_path = package_name.replace('.', os.path.sep)
    for _, name, is_package in pkgutil.iter_modules([package_path]):
        child_name = package_name + '.' + name
        if is_package:
            for _module in iter_package(child_name, filter=filter):
                yield _module
        else:
            if filter(name):
                yield importlib.import_module(child_name)


is_handler = lambda _name: _name.endswith('handler')
is_api = lambda _name: _name.endswith('controller') or _name.endswith('api')
is_rpc = lambda _name: _name.endswith('rpc')
is_model = lambda _name: _name.endswith('model')


def iter_packages(packages, filter=lambda _name: True):
    for package in packages:
        for _module in iter_package(package, filter):
            yield _module


iter_handler_packages = functools.partial(iter_packages, filter=is_handler)
iter_api_packages = functools.partial(iter_packages, filter=is_api)
iter_rpc_packages = functools.partial(iter_packages, filter=is_rpc)
iter_model_packages = functools.partial(iter_packages, filter=is_model)


def import_packages(packages):
    iter_handler = iter_packages(packages)
    return [item for item in iter_handler]


def import_handler_packages(packages):
    iter_handler = iter_handler_packages(packages)
    return [item for item in iter_handler]


def import_api_packages(packages):
    iter_api = iter_api_packages(packages)
    return [item for item in iter_api]


def import_rpc_packages(packages):
    iter_rpc = iter_rpc_packages(packages)
    return [item for item in iter_rpc]


def import_model_packages(packages):
    iter_model = iter_model_packages(packages)
    return [item for item in iter_model]



# -*- coding: utf-8 -*-
__author__ = 'XQ'

# !/usr/bin/env python

#############################################
# File Name: setup.py
# Author: XQ
# Mail: 5273508@qq.com
# Created Time:  2020-04-07
#############################################

from setuptools import setup, find_packages  # 这个包没有的可以pip一下

setup(
    name="xq-common",  # 这里是pip项目发布的名称
    version="1.0.0.0",  # 版本号，数值大的会优先被pip
    keywords=("pip", "common"),
    description="xq common",
    long_description="",
    license="MIT Licence",
    url="",  # 项目相关文件地址，一般是github
    author="XQ",
    author_email="5273508@qq.com",
    packages=find_packages(),
    include_package_data=True,
    platforms='python 3.7',
    install_requires=[],  # 这个项目需要的第三方库
)
